import { onDestroy } from 'svelte';
import { MutationObserver, notifyManager } from '@tanstack/query-core';
import { useQueryClient } from './useQueryClient';
export function createMutation(options, queryClient) {
    const client = useQueryClient(queryClient);
    const observer = $derived(new MutationObserver(client, options));
    const mutate = $state((variables, mutateOptions) => {
        observer.mutate(variables, mutateOptions).catch(noop);
    });
    $effect.pre(() => {
        observer.setOptions(options);
    });
    let result = $state(observer.getCurrentResult());
    const un = observer.subscribe((val) => {
        notifyManager.batchCalls(() => {
            result = val;
        })();
    });
    onDestroy(un);
    const data = $state({
        ...result,
        mutate,
        mutateAsync: result.mutate,
    });
    $effect.pre(() => {
        Object.assign(data, {
            ...result,
            mutate,
            mutateAsync: result.mutate,
        });
    });
    return data;
}
// eslint-disable-next-line @typescript-eslint/no-empty-function
function noop() { }
